/* chpath -- substitute a path in a sysvar for another path
   Copyright (C) 1999 Darren Salt.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:
   1. Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
   3. All advertising materials mentioning features or use of this software
      must display the following acknowledgement:
        This product includes software developed by Darren Salt.
   4. The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

   This software is provided by the author "as is" and any express or implied
   warranties including, but not limited to, the implied warranties of
   merchantability and fitness for a particular purpose are disclaimed. in no
   event shall the author be liable for any direct, indirect, incidental,
   special, exemplary, or consequential damages (including, but not limited
   to, procurement of substitute goods or services; loss of use, data, or
   profits; or business interruption) however caused and on any theory of
   liability, whether in contract, strict liability, or tort (including
   negligence or otherwise) arising in any way out of the use of this
   software, even if advised of the possibility of such damage.
 */

/* Written by Darren Salt <ds@youmustbejoking.demon.co.uk>.  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "var.h"

int
main (int argc, char *argv[])
{
  int vlen, voldlen, vnewlen;
  char *vbuf, *voffset;

  if (argc != 4) {
    printf ("Syntax: chpath <path-variable> <old-path-component> <new-path-component>\n");
    return 1;
  }
  vlen = ~varlen (argv[1]);
  if (vlen < 0) {
    printf ("chpath: system variable %s not found\n", argv[1]);
    return 1;
  }
  voldlen = strlen (argv[2]);
  vnewlen = strlen (argv[3]);
  vbuf = malloc (vlen + 1 + (vnewlen - voldlen > 0 ? vnewlen - voldlen : 0));
  if (!vbuf) {
    printf ("chpath: out of memory\n");
    return 1;
  }
  readvar (argv[1], vbuf, vlen + 1);
  vbuf[vlen] = 0;
  voffset = vbuf;
  for (;;) {
    if (!strncmp (voffset, argv[2], voldlen) &&
        (voffset[voldlen] == ',' || voffset[voldlen] == 0))
      break;
    voffset = strchr (voffset, ',');
    if (!voffset++)
      return 0;
  }

  memmove (voffset + vnewlen, voffset + voldlen,
	   vlen + 1 - (voffset - vbuf) - voldlen);
  memcpy (voffset, argv[3], vnewlen);

  setvar (argv[1], vbuf, strlen (vbuf));
  return 0;
}
